@@ -209,7 +209,7 @@ spui_settings_response (GtkDialog *dialog,
 	    spui_apply_sensitivity (FALSE);
         break;
         case GTK_RESPONSE_HELP:
-	    gn_load_help ("gnopernicus-speech-settings");
+	    gn_load_help ("gnopernicus-speech-prefs");
 	break;
 	default:
 	    gtk_widget_hide ((GtkWidget*)dialog);
