@@ -40,22 +40,44 @@
 
 #define STR(X) (X == NULL? "" : X)
 
-/* FIXME:next 4 variables must be static */
-SRCSpeechTextEchoType  	   src_speech_text_echo_type     = SRC_SPEECH_TEXT_ECHO_CHAR;
-SRCSpeechModifiersEchoType src_speech_modifiers_echo_type= SRC_SPEECH_MODIFIERS_ECHO_ALL;
-SRCSpeechSpacesEchoType    src_speech_spaces_echo_type   = SRC_SPEECH_SPACES_ECHO_ALL;
-SRCSpeechCursorsEchoType   src_speech_cursors_echo_type  = SRC_SPEECH_CURSORS_ECHO_ALL;
-SRCSpeechDictionaryType    src_speech_dictionary_type    = SRC_SPEECH_DICTIONARY_YES;
-
-static SRCSpeechSpellingMode      src_speech_spelling_mode      = SRC_SPEECH_SPELL_NORMAL;
-static SRCSpeechCountMode     	  src_speech_count_mode         = SRC_SPEECH_COUNT_NONE;
-static gchar 	**src_speech_engine_voices = NULL;
-static gchar 	**src_speech_engine_drivers= NULL;
-static gchar 	**src_speech_voices        = NULL;
 static gboolean src_speech_initialized     = FALSE;
 
 static GHashTable *src_dictionary_hash 	= NULL;
 
+typedef struct 
+{
+    SRCSpeechCountMode     	  count_mode;
+    SRCSpeechSpellingMode      	  spelling_mode;
+    SRCSpeechTextEchoType  	  text_echo_type;
+    SRCSpeechModifiersEchoType 	  modifiers_echo_type;
+    SRCSpeechSpacesEchoType    	  spaces_echo_type;
+    SRCSpeechCursorsEchoType   	  cursors_echo_type;
+    SRCSpeechDictionaryType    	  dictionary_type;
+    
+}SRCSpeech;
+
+typedef struct
+{
+     gchar 	**engine_voices;
+     gchar 	**engine_drivers;
+     gchar 	**voices;
+
+}SRCVoices;
+
+typedef struct
+{
+    gchar *voice;
+    gchar *driver;
+    gchar *id;
+    gchar *rate;
+    gchar *pitch;
+    gchar *volume;
+
+}SRCVoice;
+
+static GPtrArray     	*src_voice_params = NULL;
+SRCSpeech 	 	*src_speech = NULL;
+static SRCVoices 	*src_voices = NULL;
 
 static gchar**
 src_speech_get_engine_drivers ()
@@ -109,7 +131,7 @@ src_speech_get_engine_voices ()
     GSList *tmp2    = NULL;
 
     sru_assert (src_speech_initialized);
-    sru_assert (src_speech_engine_drivers);
+    sru_assert (src_voices->engine_drivers);
     
 
     srconf_get_data_with_default (SPEECH_VOICE_COUNT, CFGT_INT, 
@@ -123,9 +145,9 @@ src_speech_get_engine_voices ()
     index2 = 0;
     iter = 0;
 
-    for (iter = 0; src_speech_engine_drivers[iter] ; iter++)
+    for (iter = 0; src_voices->engine_drivers[iter] ; iter++)
     {
-	srconf_get_data_with_default (src_speech_engine_drivers[iter], CFGT_LIST, 
+	srconf_get_data_with_default (src_voices->engine_drivers[iter], CFGT_LIST, 
 				      &voices, NULL, SPEECH_DRIVERS_SECTION);
 	for (tmp2 = voices; tmp2 ; tmp2 = tmp2->next)
 	{
@@ -154,22 +176,22 @@ src_speech_valid_engine_voice ( gchar *driver,
     gboolean valid_driver = FALSE;
 
     sru_assert (src_speech_initialized);
-    sru_assert (src_speech_engine_voices);
-    sru_assert (src_speech_engine_drivers);
+    sru_assert (src_voices->engine_voices);
+    sru_assert (src_voices->engine_drivers);
     sru_assert (voice);
 
-    for (i = 0; src_speech_engine_drivers[i]; i++)
+    for (i = 0; src_voices->engine_drivers[i]; i++)
     {
-	if (strcmp (driver, src_speech_engine_drivers[i]) == 0)
+	if (strcmp (driver, src_voices->engine_drivers[i]) == 0)
 	    valid_driver = TRUE;
     }
 
     if (!valid_driver)
 	return FALSE;
 	
-    for (i = 0; src_speech_engine_voices[i]; i++)
+    for (i = 0; src_voices->engine_voices[i]; i++)
     {
-	if (strcmp (voice, src_speech_engine_voices[i]) == 0)
+	if (strcmp (voice, src_voices->engine_voices[i]) == 0)
 	    return TRUE;
     }
     return FALSE;
@@ -221,12 +243,12 @@ src_speech_valid_voice (gchar *voice)
     gint i;
 
     sru_assert (src_speech_initialized);
-    sru_assert (src_speech_voices);
+    sru_assert (src_voices->voices);
     sru_assert (voice);
 
-    for (i = 0; src_speech_voices[i]; i++)
+    for (i = 0; src_voices->voices[i]; i++)
     {
-	if (strcmp (voice, src_speech_voices[i]) == 0)
+	if (strcmp (voice, src_voices->voices[i]) == 0)
 	    return TRUE;
     }    
     return FALSE;
@@ -245,7 +267,88 @@ src_speech_valid_voice (gchar *voice)
 		    *string = NULL;					      \
 		}							      \
 	    }
+static SRCVoice* 
+src_voice_new ()
+{
+    return g_new0 (SRCVoice, 1);
+}
 
+static gboolean
+src_speech_get_voice_parameters_from_gconf (gchar *voice)
+{
+    GSList *voices = NULL;
+    SRCVoice *voice_ = NULL;
+
+    sru_assert (src_speech_initialized);
+    sru_assert (voice);
+
+    voice_ = src_voice_new ();
+    srconf_get_data_with_default (voice, CFGT_LIST, &voices, 
+				  NULL, SPEECH_PARAMETER_SECTION);
+    
+    if (voices)
+    {
+	GSList *crt = voices;
+	
+	voice_->voice  = g_strdup (voice);
+	voice_->driver = g_strdup (crt->data);
+	if (crt)
+	    crt = crt->next;
+	voice_->id = g_strdup (crt->data);
+
+	if (crt)
+	    crt = crt->next;
+	voice_->volume = g_strdup (crt->data);
+
+	if (crt)
+	    crt = crt->next;
+	voice_->rate = g_strdup (crt->data);
+	
+	if (crt)
+	    crt = crt->next;
+	voice_->pitch = g_strdup (crt->data);
+	
+	srconf_free_slist (voices);
+	g_ptr_array_add (src_voice_params, voice_);
+    }
+    else
+    {
+	voice_->driver 	= NULL;
+	voice_->id 	= NULL;
+	voice_->volume 	= NULL;
+	voice_->pitch 	= NULL;
+	voice_->rate 	= NULL;
+	
+    }
+    return TRUE;
+}
+
+static void
+src_speech_voices_parameters_free ()
+{
+    g_ptr_array_free (src_voice_params, TRUE);
+}
+
+static gboolean
+src_speech_update_voices_parameters ()
+{
+    gchar ** voices;
+    
+    sru_assert (src_speech_initialized);
+    
+    src_speech_voices_parameters_free ();
+    src_voice_params = g_ptr_array_new ();
+
+    voices = src_voices->voices;
+    if (voices)
+    {
+	gint i;
+	for (i = 0; voices[i]; i++)
+	   src_speech_get_voice_parameters_from_gconf (voices[i]);
+    }
+    
+    return TRUE;
+}
 
 static gboolean
 src_speech_get_voice_parameters (gchar *voice,
@@ -255,68 +358,102 @@ src_speech_get_voice_parameters (gchar *voice,
 				 gchar **rate,
 				 gchar **pitch)
 {
-    GSList *voices = NULL;
+    SRCVoice *voice_ = NULL;
+    gint i;    
 
     sru_assert (src_speech_initialized);
-    sru_assert (voice && voice[0]);
-
-    srconf_get_data_with_default (voice, CFGT_LIST, &voices, 
-					    NULL, SPEECH_PARAMETER_SECTION);
-
-    if (voices)
+    sru_assert (voice && src_voice_params);
+    
+    for (i = 0; i < src_voice_params->len; i++)
     {
-	GSList *crt = voices;
-	
-	if (driver)
-	    src_speech_set_string_from_list_element (crt, driver);
-	if (crt)
-	    crt = crt->next;
-	if (name)
-	    src_speech_set_string_from_list_element (crt, name);
-	if (crt)
-	    crt = crt->next;
-	if (volume)
-	    src_speech_set_string_from_list_element (crt, volume);
-	if (crt)
-	    crt = crt->next;
-	if (rate)
-	    src_speech_set_string_from_list_element (crt, rate);
-	if (crt)
-	    crt = crt->next;
-	if (pitch)
-	    src_speech_set_string_from_list_element (crt, pitch);
-	srconf_free_slist (voices);
+	voice_ = g_ptr_array_index (src_voice_params, i);
+
+	if (voice_)
+	{
+	    if (strcmp (voice_->voice, voice) == 0)
+	    {
+		    if (driver)
+			*driver = g_strdup (voice_->driver);
+		    if (name)
+			*name   = g_strdup (voice_->id);
+		    if (volume)
+			*volume = g_strdup (voice_->volume);
+		    if (rate)
+			*rate   = g_strdup (voice_->rate);
+		    if (pitch)
+			*pitch  = g_strdup (voice_->pitch);
+
+		break;
+	    }
+	}
     }
-    else
-    {
-	if (driver)
-	    *driver = NULL;
-	if (name)
-	    *name = NULL;
-	if (volume)
-	    *volume = NULL;
-	if (rate)
-	    *rate = NULL;
-	if (pitch)
-	    *pitch = NULL;
-    }
-    /* fprintf (stderr, "\ngotten parameters for voice: %s are name: %s volume: %s rate: %s pitch: %s",
-		    voice,
-		    *name ? *name : "(null)",
-		    *volume ? *volume : "(null)",
-		    *rate ? *rate : "(null)",
-		    *pitch ? *pitch : "(null)"); */
+
     return TRUE;
 }
 
-
 static gboolean
 src_speech_set_voice_parameters (gchar *voice,
-				 gchar *driver_,
-				 gchar *name_,
-				 gchar *volume_,
-				 gchar *rate_,
-				 gchar *pitch_)
+				 gchar *driver,
+				 gchar *name,
+				 gchar *volume,
+				 gchar *rate,
+				 gchar *pitch)
+{
+    SRCVoice *voice_ = NULL;
+    gint i;
+    
+    sru_assert (src_speech_initialized);
+    sru_assert (voice);
+
+    for (i = 0; i < src_voice_params->len; i++)
+    {
+	voice_ = g_ptr_array_index (src_voice_params, i);
+
+	if (voice_)
+	{
+	    if (strcmp (voice_->voice, voice) == 0)
+	    {
+		
+		if (driver)
+		{
+		    g_free (voice_->driver);
+		    voice_->driver 	= g_strdup (driver);
+		}
+		if (name)
+		{
+		    g_free (voice_->id);
+		    voice_->id 		= g_strdup (name);
+		}
+		if (volume)
+		{
+		    g_free (voice_->volume);
+		    voice_->volume 	= g_strdup (volume);
+		}
+		if (rate)
+		{
+		    g_free (voice_->rate);
+		    voice_->rate 	= g_strdup (rate);
+		}
+		if (pitch)
+		{
+		    g_free (voice_->pitch);
+		    voice_->pitch 	= g_strdup (pitch);
+		}
+		break;
+	    }
+	}
+    }
+
+    return TRUE;
+}
+
+static gboolean
+src_speech_set_voice_parameters_to_gconf (gchar *voice,
+					  gchar *driver_,
+					  gchar *name_,
+					  gchar *volume_,
+					  gchar *rate_,
+					  gchar *pitch_)
 {
     gchar *driver, *name, *volume, *rate, *pitch;
     gboolean rv = FALSE;
@@ -324,6 +461,7 @@ src_speech_set_voice_parameters (gchar *voice,
     sru_assert (src_speech_initialized);
     sru_assert (voice && voice[0]);
 
+    driver = name = volume = rate = pitch = NULL;
     src_speech_get_voice_parameters (voice, &driver, &name, &volume, &rate, &pitch);
 
     if (driver_)
@@ -366,9 +504,11 @@ src_speech_set_voice_parameters (gchar *voice,
 	list = g_slist_append (list, pitch);
 	srconf_set_data (voice, CFGT_LIST, list, SPEECH_PARAMETER_SECTION);
 	srconf_free_slist (list);
+	src_speech_set_voice_parameters (voice, driver_, name_, volume_,
+					 rate_, pitch_);
 	rv = TRUE;
     }    
-
+    
     return rv;
 }
 
@@ -471,18 +611,20 @@ src_speech_init_voice (gchar *voice)
     driver = name = volume = rate = pitch = NULL;
     if (src_speech_get_voice_parameters (voice, &driver, &name, &volume, &rate, &pitch))
     {
+	
 	if (!name || !driver ||
 	    !src_speech_valid_engine_voice (driver, name))
 	{
 	    g_free (name);
 	    g_free (driver);
-	    name = g_strdup (src_speech_engine_voices[0]);
-	    driver = g_strdup (src_speech_engine_drivers[0]);
-	    src_speech_set_voice_parameters (voice, driver, name, volume, rate, pitch);
+	    name = g_strdup (src_voices->engine_voices[0]);
+	    driver = g_strdup (src_voices->engine_drivers[0]);
+	    src_speech_set_voice_parameters_to_gconf (voice, driver, name, volume, rate, pitch);
 	}
+	
+	
 	rv = src_speech_create_voice (voice, driver, name, volume, rate, pitch);
     }
-
     g_free (driver);
     g_free (name);
     g_free (volume);
@@ -541,17 +683,17 @@ src_speech_add_voice (gchar *voice)
 
     sru_assert (src_speech_initialized);
     sru_assert (voice);
-    sru_assert (src_speech_voices);
+    sru_assert (src_voices->voices);
 
-    for (i = 0; src_speech_voices[i]; i++)
-	if (strcmp (voice, src_speech_voices[i]) == 0)
+    for (i = 0; src_voices->voices[i]; i++)
+	if (strcmp (voice, src_voices->voices[i]) == 0)
 	    return TRUE;
-    voices = src_speech_voices;
-    src_speech_voices = g_new (gchar*, i + 2);
+    voices = src_voices->voices;
+    src_voices->voices = g_new (gchar*, i + 2);
     for (i = 0; voices[i]; i++)
-	src_speech_voices[i] = voices[i];
-    src_speech_voices[i] = g_strdup (voice);
-    src_speech_voices[i + 1] = NULL;
+	src_voices->voices[i] = voices[i];
+    src_voices->voices[i] = g_strdup (voice);
+    src_voices->voices[i + 1] = NULL;
     g_free (voices);
 
     return TRUE;
@@ -566,15 +708,15 @@ src_speech_remove_voice (gchar *voice)
 
     sru_assert (src_speech_initialized);
     sru_assert (voice);
-    sru_assert (src_speech_voices);
+    sru_assert (src_voices->voices);
 
-    for (i = 0; src_speech_voices[i]; i++)
-	if (strcmp (voice, src_speech_voices[i]) == 0)
+    for (i = 0; src_voices->voices[i]; i++)
+	if (strcmp (voice, src_voices->voices[i]) == 0)
 	    break;
-    if (src_speech_voices[i])
+    if (src_voices->voices[i])
 	rv = TRUE;
-    for ( ; src_speech_voices[i]; i++)
-	src_speech_voices[i] = src_speech_voices[i + 1];
+    for ( ; src_voices->voices[i]; i++)
+	src_voices->voices[i] = src_voices->voices[i + 1];
 
     return rv;
 }
@@ -588,7 +730,7 @@ src_speech_test_voice (gchar *voice)
 
     sru_assert (src_speech_initialized);
     sru_assert (voice && voice[0]);
-    sru_assert (src_speech_voices);
+    sru_assert (src_voices->voices);
     
     if (!src_speech_add_voice ("test"))
 	return FALSE;
@@ -598,7 +740,7 @@ src_speech_test_voice (gchar *voice)
     {
 	if (name && driver && 
 	    src_speech_valid_engine_voice (driver, name))
-	    rv = src_speech_create_voice ("test", driver, name, volume, rate, pitch);
+	    rv = src_speech_create_voice ("test", driver, name, volume, rate, pitch);	    
     }
 
     if (rv)
@@ -629,14 +771,17 @@ src_speech_voices_init ()
 
     sru_assert (src_speech_initialized);
 
-    voices = src_speech_voices;
+    voices = src_voices->voices;
     if (voices)
     {
 	gint i;
 	for (i = 0; voices[i]; i++)
-	    src_speech_init_voice (voices[i]);
+	{
+	   src_speech_get_voice_parameters_from_gconf (voices[i]);
+	   src_speech_init_voice (voices[i]);
+	}
     }
-
+    
     return TRUE;
 }
 
@@ -654,7 +799,7 @@ src_speech_set_spelling_mode (SRCSpeechSpellingMode mode)
     sru_assert (src_speech_initialized);
     sru_assert (mode  != SRC_SPEECH_SPELL_AUTO);
 
-    src_speech_spelling_mode = mode;
+    src_speech->spelling_mode = mode;
 
     return TRUE;
 }
@@ -665,7 +810,7 @@ src_speech_get_spelling_mode ()
 {
     sru_assert (src_speech_initialized);
 
-    return src_speech_spelling_mode;
+    return src_speech->spelling_mode;
 }
 
 
@@ -675,7 +820,7 @@ src_speech_set_count_mode (SRCSpeechCountMode mode)
     sru_assert (src_speech_initialized);
     sru_assert (mode  != SRC_SPEECH_COUNT_AUTO);
 
-    src_speech_count_mode = mode;
+    src_speech->count_mode = mode;
     
     return TRUE;
 }
@@ -685,7 +830,7 @@ static gboolean
 src_speech_set_count_mode_from_string (gchar *mode)
 {
     gboolean rv = TRUE;
-    SRCSpeechCountMode mode_ = src_speech_count_mode;
+    SRCSpeechCountMode mode_ = src_speech->count_mode;
 
     sru_assert (mode);
 
@@ -711,7 +856,7 @@ src_speech_set_modifiers_echo_type (SRCSpeechModifiersEchoType type)
 {
     sru_assert (src_speech_initialized);
 
-    src_speech_modifiers_echo_type = type;
+    src_speech->modifiers_echo_type = type;
     
     return TRUE;
 }
@@ -721,7 +866,7 @@ static gboolean
 src_speech_set_modifiers_echo_type_from_string (gchar *type)
 {
     gboolean rv = TRUE;
-    SRCSpeechModifiersEchoType type_ = src_speech_modifiers_echo_type;
+    SRCSpeechModifiersEchoType type_ = src_speech->modifiers_echo_type;
 
     sru_assert (type);
 
@@ -747,7 +892,7 @@ src_speech_set_cursors_echo_type (SRCSpeechCursorsEchoType type)
 {
     sru_assert (src_speech_initialized);
 
-    src_speech_cursors_echo_type = type;
+    src_speech->cursors_echo_type = type;
     
     return TRUE;
 }
@@ -757,7 +902,7 @@ static gboolean
 src_speech_set_cursors_echo_type_from_string (gchar *type)
 {
     gboolean rv = TRUE;
-    SRCSpeechCursorsEchoType type_ = src_speech_cursors_echo_type;
+    SRCSpeechCursorsEchoType type_ = src_speech->cursors_echo_type;
 
     sru_assert (type);
 
@@ -783,7 +928,7 @@ src_speech_set_spaces_echo_type (SRCSpeechSpacesEchoType type)
 {
     sru_assert (src_speech_initialized);
 
-    src_speech_spaces_echo_type = type;
+    src_speech->spaces_echo_type = type;
     
     return TRUE;
 }
@@ -793,7 +938,7 @@ static gboolean
 src_speech_set_spaces_echo_type_from_string (gchar *type)
 {
     gboolean rv = TRUE;
-    SRCSpeechSpacesEchoType type_ = src_speech_spaces_echo_type;
+    SRCSpeechSpacesEchoType type_ = src_speech->spaces_echo_type;
 
     sru_assert (type);
 
@@ -819,7 +964,7 @@ src_speech_set_dictionary_type (SRCSpeechDictionaryType type)
 {
     sru_assert (src_speech_initialized);
 
-    src_speech_dictionary_type = type;
+    src_speech->dictionary_type = type;
     
     return TRUE;
 }
@@ -829,7 +974,7 @@ static gboolean
 src_speech_set_dictionary_type_from_string (gchar *type)
 {
     gboolean rv = TRUE;
-    SRCSpeechDictionaryType type_ = src_speech_dictionary_type;
+    SRCSpeechDictionaryType type_ = src_speech->dictionary_type;
 
     sru_assert (type);
 
@@ -854,7 +999,7 @@ src_speech_set_text_echo_type (SRCSpeechTextEchoType type)
 {
     sru_assert (src_speech_initialized);
     
-    src_speech_text_echo_type = type;
+    src_speech->text_echo_type = type;
     
     return TRUE;
 }
@@ -864,7 +1009,7 @@ static gboolean
 src_speech_set_text_echo_type_from_string (gchar *type)
 {
     gboolean rv = TRUE;
-    SRCSpeechTextEchoType type_ = src_speech_text_echo_type;
+    SRCSpeechTextEchoType type_ = src_speech->text_echo_type;
 
     sru_assert (type);
 
@@ -1187,14 +1332,15 @@ src_speech_init_default_voice ()
     gchar *driver, *name, *volume, *rate, *pitch;
  
     sru_assert (src_speech_initialized);
-    sru_assert (src_speech_engine_voices);
+    sru_assert (src_voices->engine_voices);
    
-    driver = g_strdup (src_speech_engine_drivers[0]);
-    name   = g_strdup (src_speech_engine_voices[0]);
+    driver = g_strdup (src_voices->engine_drivers[0]);
+    name   = g_strdup (src_voices->engine_voices[0]);
     volume = g_strdup_printf ("%d", DEFAULT_SPEECH_VOLUME);
     rate   = g_strdup_printf ("%d", DEFAULT_SPEECH_RATE);
     pitch  = g_strdup_printf ("%d", DEFAULT_SPEECH_PITCH);
-    rv = src_speech_set_voice_parameters ("default", driver, name, volume, rate, pitch);
+    rv = src_speech_set_voice_parameters_to_gconf ("default", driver, name, volume, rate, pitch);
+    
     g_free (driver);
     g_free (name);
     g_free (rate);
@@ -1277,25 +1423,65 @@ src_speech_load_defaults ()
     return TRUE;
 }
 
+static SRCSpeech*
+src_speech_src_speech_new ()
+{
+    SRCSpeech *speech;
+    
+    speech = g_new0 (SRCSpeech, 1);
+    
+    return speech;
+}
+
+static SRCVoices*
+src_speech_src_voices_new ()
+{
+    SRCVoices *voices;
+    
+    voices = g_new0 (SRCVoices, 1);
+    
+    return voices;
+}
+
+void
+src_speech_set_default_values (SRCSpeech *src_speech)
+{
+    src_speech->text_echo_type     = SRC_SPEECH_TEXT_ECHO_CHAR;
+    src_speech->modifiers_echo_type= SRC_SPEECH_MODIFIERS_ECHO_ALL;
+    src_speech->spaces_echo_type   = SRC_SPEECH_SPACES_ECHO_ALL;
+    src_speech->cursors_echo_type  = SRC_SPEECH_CURSORS_ECHO_ALL;
+    src_speech->dictionary_type    = SRC_SPEECH_DICTIONARY_YES;
+    src_speech->spelling_mode      = SRC_SPEECH_SPELL_NORMAL;
+    src_speech->count_mode         = SRC_SPEECH_COUNT_NONE;
+}
 
 gboolean
 src_speech_init ()
 {
     sru_assert (!src_speech_initialized);
     
+    src_voice_params = g_ptr_array_new ();
+    
+    src_voices = src_speech_src_voices_new ();
+    src_speech = src_speech_src_speech_new ();
+    
+    src_speech_set_default_values (src_speech);
+    
     src_speech_initialized = srs_xml_init (NULL);
 
     if (src_speech_initialized)
     {
-	src_speech_engine_drivers= src_speech_get_engine_drivers ();
-	src_speech_engine_voices = src_speech_get_engine_voices  ();
-	src_speech_voices 	 = src_speech_get_voices ();
+	src_voices->engine_drivers= src_speech_get_engine_drivers ();
+	src_voices->engine_voices = src_speech_get_engine_voices  ();
+	src_voices->voices 	  = src_speech_get_voices ();
     }
     
-    if (src_speech_initialized && src_speech_engine_voices && src_speech_voices)
+    if (src_speech_initialized && 
+     src_voices->engine_voices && 
+     src_voices->voices)
     {
 	src_speech_initialized = src_speech_init_default_voice () &&
-						    src_speech_voices_init ();
+				 src_speech_voices_init ();
     }
     else
     {
@@ -1325,8 +1511,8 @@ src_speech_init ()
     }
     if (!src_speech_initialized)
     {
-	g_strfreev (src_speech_voices);
-	g_strfreev (src_speech_engine_voices);
+	g_strfreev (src_voices->voices);
+	g_strfreev (src_voices->engine_voices);
     }
 
     return src_speech_initialized;
@@ -1342,9 +1528,9 @@ src_speech_terminate ()
     src_speech_voices_terminate ();
     src_dictionary_terminate ();
     src_speech_initialized = FALSE;    
-    g_strfreev (src_speech_voices);
-    g_strfreev (src_speech_engine_voices);
-    g_strfreev (src_speech_engine_drivers);
+    g_strfreev (src_voices->voices);
+    g_strfreev (src_voices->engine_voices);
+    g_strfreev (src_voices->engine_drivers);
 }
 
 #define src_speech_is_separator(x) (( x == '.' || x == ';' || x == '?' || x == ',' || x == '!' ) ? TRUE : FALSE)
@@ -1502,9 +1688,9 @@ src_speech_process_string (const gchar   *text,
     sru_assert (text);
 
     if (cmode == SRC_SPEECH_COUNT_AUTO)
-	cmode = src_speech_count_mode;
+	cmode = src_speech->count_mode;
     if (smode == SRC_SPEECH_SPELL_AUTO)
-	smode = src_speech_spelling_mode;
+	smode = src_speech->spelling_mode;
 
     if (!src_enable_format_string)
 	return g_strdup (text);
@@ -1523,7 +1709,7 @@ src_speech_process_string (const gchar   *text,
 	    return g_strdup ((gchar*)text);
     }
 
-    if (src_speech_dictionary_type == SRC_SPEECH_DICTIONARY_YES)
+    if (src_speech->dictionary_type == SRC_SPEECH_DICTIONARY_YES)
     {
 	crt = src_dictionary_process_string (crt);
 	tmp = crt;
@@ -1566,7 +1752,7 @@ src_speech_process_string (const gchar   *text,
     }
     
     
-    if (src_speech_dictionary_type == SRC_SPEECH_DICTIONARY_YES)
+    if (src_speech->dictionary_type == SRC_SPEECH_DICTIONARY_YES)
 	g_free (tmp);
     	
     return rv[0] ? rv : NULL;
@@ -1584,7 +1770,7 @@ src_speech_get_text_voice_attributes (gchar *voice,
     sru_assert (voice);
     
     if (smode == SRC_SPEECH_SPELL_AUTO)
-	mode = src_speech_spelling_mode;
+	mode = src_speech->spelling_mode;
 
     if (mode == SRC_SPEECH_SPELL_MILITARY)
 	spelling = " spelling='military'";
@@ -1621,7 +1807,7 @@ src_speech_modify_pitch (SRCModifyMode mode)
 	
     src_speech_say_message (message);
 
-    voices = src_speech_voices;
+    voices = src_voices->voices;
     if (voices)
     {
 	gint i;
@@ -1643,7 +1829,7 @@ src_speech_modify_pitch (SRCModifyMode mode)
 
 	    pitch = MIN (MAX (pitch, MIN_SPEECH_PITCH), MAX_SPEECH_PITCH);
 	    pitch_ = g_strdup_printf ("%d", pitch);
-	    if (src_speech_set_voice_parameters (voices[i], NULL, NULL, NULL, NULL, pitch_))
+	    if (src_speech_set_voice_parameters_to_gconf (voices[i], NULL, NULL, NULL, NULL, pitch_))
 		src_speech_create_voice (voices[i], NULL, NULL, NULL, NULL, pitch_);
 	    g_free (pitch_);	    
 	}
@@ -1670,7 +1856,7 @@ src_speech_modify_rate (SRCModifyMode mode)
 	
     src_speech_say_message (message);
 
-    voices = src_speech_voices;
+    voices = src_voices->voices;
     if (voices)
     {
 	gint i;
@@ -1692,7 +1878,7 @@ src_speech_modify_rate (SRCModifyMode mode)
 
 	    rate = MIN (MAX (rate, MIN_SPEECH_RATE), MAX_SPEECH_RATE);
 	    rate_ = g_strdup_printf ("%d", rate);
-	    if (src_speech_set_voice_parameters (voices[i], NULL, NULL, NULL, rate_, NULL))
+	    if (src_speech_set_voice_parameters_to_gconf (voices[i], NULL, NULL, NULL, rate_, NULL))
 		src_speech_create_voice (voices[i], NULL, NULL, NULL, rate_, NULL);
 	    g_free (rate_);	    
 	}
@@ -1720,7 +1906,7 @@ src_speech_modify_volume (SRCModifyMode mode)
 	
     src_speech_say_message (message);
 
-    voices = src_speech_voices;
+    voices = src_voices->voices;
     if (voices)
     {
 	gint i;
@@ -1742,7 +1928,7 @@ src_speech_modify_volume (SRCModifyMode mode)
 
 	    volume = MIN (MAX (volume, MIN_SPEECH_VOLUME), MAX_SPEECH_VOLUME);
 	    volume_ = g_strdup_printf ("%d", volume);
-	    if (src_speech_set_voice_parameters (voices[i], NULL, NULL, volume_, NULL, NULL))
+	    if (src_speech_set_voice_parameters_to_gconf (voices[i], NULL, NULL, volume_, NULL, NULL))
 		src_speech_create_voice (voices[i], NULL, NULL, volume_, NULL, NULL);
 	    g_free (volume_);	    
 	}
@@ -1764,8 +1950,10 @@ src_speech_process_voice_config_changed (SRConfigStructure *config)
     if (!strcmp (config->key, SPEECH_GNOPERNICUS_SPEAKERS))
 	    return rv;
 	
-    rv = src_speech_add_voice (config->key) &&
-	    src_speech_init_voice (config->key);
+    rv = src_speech_add_voice (config->key)  &&
+	 src_speech_init_voice (config->key) &&
+	 src_speech_update_voices_parameters();
+    
     
     return rv;
 }
@@ -1872,4 +2060,27 @@ src_process_string (const gchar   *text,
     return src_speech_process_string (text, cmode, smode);
 }
 
+SRCSpeechTextEchoType
+src_speech_get_text_echo_type ()
+{
+    return src_speech->text_echo_type;
+}
+
+SRCSpeechSpacesEchoType
+src_speech_get_spaces_echo_type ()
+{
+    return src_speech->spaces_echo_type;
+}
+
+SRCSpeechModifiersEchoType
+src_speech_get_modifiers_echo_type ()
+{
+    return src_speech->modifiers_echo_type;
+}
+
+SRCSpeechCursorsEchoType
+src_speech_get_cursors_echo_type ()
+{
+    return src_speech->cursors_echo_type;
+}
 /*FIXME end*/
