@@ -69,6 +69,10 @@
 #define SRL_EVENT_NAME_ACTIVE_DESCENDANT_CHANGED "object:active-descendant-changed"
 #define SRL_EVENT_NAME_NAME_CHANGED		"object:property-change:accessible-name"
 #define SRL_EVENT_NAME_VISIBLE_DATA_CHANGED	"object:visible-data-changed"
+#define SRL_EVENT_NAME_CHILDREN_ADDED           "object:children-changed:add"
+#define SRL_EVENT_NAME_CHILDREN_REMOVED         "object:children-changed:remove"
+#define SRL_EVENT_NAME_TAB_ADDED                "object:tab-added"
+#define SRL_EVENT_NAME_TAB_REMOVED              "object:tab-removed"
 
 #define SRL_TOOLKIT_MOZILLA			"mozilla"
 #define SRL_APP_METACITY			"metacity"
@@ -107,6 +111,12 @@ typedef enum _SRLEventType
     SRL_EVENT_ACTIVE_DESCENDANT_CHANGED,
     SRL_EVENT_NAME_CHANGED,
     SRL_EVENT_VISIBLE_DATA_CHANGED,
+    SRL_EVENT_CHILDREN_ADDED,
+    SRL_EVENT_CHILDREN_REMOVED,
+    SRL_EVENT_TAB_ADDED,
+    SRL_EVENT_TAB_REMOVED,
+    
+
     SRL_EVENT_WINDOW_RENAME
 }SRLEventType;
 
@@ -150,7 +160,9 @@ static SRLEventTypeEventName srl_events_type_name[] =
 	{SRL_EVENT_MOUSE_ABS,			SRL_EVENT_NAME_MOUSE_ABS},
 	{SRL_EVENT_ACTIVE_DESCENDANT_CHANGED, 	SRL_EVENT_NAME_ACTIVE_DESCENDANT_CHANGED},
 	{SRL_EVENT_NAME_CHANGED,		SRL_EVENT_NAME_NAME_CHANGED},
-	{SRL_EVENT_VISIBLE_DATA_CHANGED,	SRL_EVENT_NAME_VISIBLE_DATA_CHANGED}
+	{SRL_EVENT_VISIBLE_DATA_CHANGED,	SRL_EVENT_NAME_VISIBLE_DATA_CHANGED},
+	{SRL_EVENT_CHILDREN_ADDED,              SRL_EVENT_NAME_CHILDREN_ADDED},
+	{SRL_EVENT_CHILDREN_REMOVED,            SRL_EVENT_NAME_CHILDREN_REMOVED}
     };
 
 typedef struct _SRLEvent
@@ -633,6 +645,11 @@ srle_set_acc (SRLEvent *event)
     {
 	event->acc = AccessibleActiveDescendantChangedEvent_getActiveDescendant (event->acc_ev);
     }
+    if (srle_has_type (event, SRL_EVENT_CHILDREN_ADDED) ||
+        srle_has_type (event, SRL_EVENT_CHILDREN_REMOVED))
+    {
+	event->acc = AccessibleChildChangedEvent_getChildAccessible (event->acc_ev);
+    }	
     if (srle_has_type (event, SRL_EVENT_LINK_SELECTED))
     {
 	if (srl_acc_has_toolkit (srle_get_acc (event), SRL_TOOLKIT_MOZILLA))
@@ -663,11 +680,16 @@ srle_get_reason (SRLEvent *event)
 
     srl_assert (event);
 
+    if (srle_has_type (event, SRL_EVENT_TAB_ADDED))
+	return SRL_EVENT_NAME_TAB_ADDED;
+    if (srle_has_type (event, SRL_EVENT_TAB_REMOVED))
+	return SRL_EVENT_NAME_TAB_REMOVED;		
+	
     if (srle_has_type (event, SRL_EVENT_SELECTION_CHANGED) &&
 	srle_acc_has_role (event, SPI_ROLE_COMBO_BOX))
 	return SRL_EVENT_NAME_CONTENT_CHANGED;
     if (srle_has_type (event, SRL_EVENT_WINDOW_ACTIVATE))
-	return SRL_EVENT_NAME_WINDOW_SWITCH;
+	return SRL_EVENT_NAME_WINDOW_SWITCH;		
 
     for (i = 0; i < G_N_ELEMENTS (srl_events_type_name); i++)
     	if (srle_has_type (event, srl_events_type_name[i].type))
@@ -708,7 +730,9 @@ srl_is_window_event (SRLEvent *event)
 	    event->type == SRL_EVENT_WINDOW_ACTIVATE ||
 	    event->type == SRL_EVENT_WINDOW_TITLELIZE||
 	    event->type == SRL_EVENT_WINDOW_DEACTIVATE ||
-	    event->type == SRL_EVENT_WINDOW_RENAME;
+	    event->type == SRL_EVENT_WINDOW_RENAME ||
+	    event->type == SRL_EVENT_TAB_ADDED ||
+	    event->type == SRL_EVENT_TAB_REMOVED;
 }
 
 static gboolean
@@ -1250,6 +1274,10 @@ srle_change_type (SRLEvent *event)
 	    event->type = SRL_EVENT_FOCUS;
     }
     
+    if (srle_has_type (event, SRL_EVENT_CHILDREN_ADDED))
+	event->type = SRL_EVENT_TAB_ADDED;
+    else if (srle_has_type (event, SRL_EVENT_CHILDREN_REMOVED))
+	event->type = SRL_EVENT_TAB_REMOVED;
     return TRUE;
 }
 
@@ -1550,7 +1578,39 @@ srl_event_is_reported (SRLEvent *event)
     
     if (srle_acc_has_role (event, SPI_ROLE_EDITBAR))
 	rv = TRUE;
-
+    
+    /*made the traversal in order to receive
+     "object:children-change:add" and "object:children-change:remove"
+     events for SPI_ROLE_PAGE_TAB_LIST objects
+     */
+    if (srle_has_type (event, SRL_EVENT_FOCUS) && 
+        srle_acc_has_role (event, SPI_ROLE_TERMINAL))
+    {
+	gint childrencnt = 0;		
+        Accessible *parent = Accessible_getParent (event->acc_ev->source);
+	Accessible *child = NULL;
+	
+        while (Accessible_getRole (parent) != SPI_ROLE_PAGE_TAB_LIST)
+	    parent = Accessible_getParent (parent);
+	
+	if (parent)
+	{   
+	    gint i;
+	    childrencnt = Accessible_getChildCount (parent);
+	    for (i = 0; i < childrencnt; i++)
+	    {
+		child = Accessible_getChildAtIndex (parent, i);
+	    }
+	    
+	    if (child)
+		Accessible_unref (child);
+	    Accessible_unref (parent);	
+	}
+    }
+    if ((srle_has_type (event, SRL_EVENT_TAB_ADDED) || 
+         srle_has_type (event, SRL_EVENT_TAB_REMOVED))) 
+	rv = srle_acc_has_role (event, SPI_ROLE_PAGE_TAB);
+	
     return rv;
 }
 
